@@ -60,6 +60,18 @@ DISCOVER_BACKEND_PLUGIN(FlatpakBackend)
 
 using namespace std::chrono_literals;
 using namespace Qt::StringLiterals;
+using namespace Utils;
+
+namespace Utils
+{
+
+QString copyAndFree(char *str)
+{
+    const QString ret = QString::fromUtf8(str);
+    g_free(str);
+    return ret;
+}
+}
 
 class FlatpakSource
 {
@@ -92,7 +104,7 @@ public:
 
     QString url() const
     {
-        return m_remote ? QString::fromUtf8(flatpak_remote_get_url(m_remote)) : QString();
+        return m_remote ? copyAndFree(flatpak_remote_get_url(m_remote)) : QString();
     }
 
     bool isEnabled() const
@@ -123,7 +135,7 @@ public:
 
     QString title() const
     {
-        auto ret = m_remote ? QString::fromUtf8(flatpak_remote_get_title(m_remote)) : QString();
+        auto ret = m_remote ? copyAndFree(flatpak_remote_get_title(m_remote)) : QString();
         if (flatpak_installation_get_is_user(m_installation)) {
             ret = i18nc("user denotes this as user-scoped flatpak repo", "%1 (user)", ret);
         }
@@ -265,11 +277,7 @@ static std::optional<AppStream::Metadata> metadataFromBytes(GBytes *appstreamGz,
     gconstpointer data = g_bytes_get_data(appstream, &len);
 
     AppStream::Metadata metadata;
-#if ASQ_CHECK_VERSION(0, 16, 0)
     metadata.setFormatStyle(AppStream::Metadata::FormatStyleCatalog);
-#else
-    metadata.setFormatStyle(AppStream::Metadata::FormatStyleCollection);
-#endif
     AppStream::Metadata::MetadataError error = metadata.parse(QString::fromUtf8((char *)data, len), AppStream::Metadata::FormatKindXml);
     if (error != AppStream::Metadata::MetadataErrorNoError) {
         qCWarning(LIBDISCOVER_BACKEND_FLATPAK_LOG) << "Failed to parse appstream metadata: " << error;
@@ -547,7 +555,9 @@ FlatpakResource *FlatpakBackend::getAppForInstalledRef(FlatpakInstallation *inst
         *freshResource = true;
     }
 
-    Q_ASSERT(resource->uniqueId() == idForInstalledRef(ref) || resource->uniqueId() == idForInstalledRef(ref, QStringLiteral(".desktop")));
+    Q_ASSERT_X(resource->uniqueId() == idForInstalledRef(ref) || resource->uniqueId() == idForInstalledRef(ref, QStringLiteral(".desktop")),
+               "getAppForInstalledRef",
+               flatpak_ref_format_ref_cached(FLATPAK_REF(ref)));
     return resource;
 }
 
@@ -740,16 +750,16 @@ QString composeRef(bool isRuntime, const QString &name, const QString &branch)
 
 AppStream::Component fetchComponentFromRemote(const QSettings &settings, GCancellable *cancellable)
 {
-    const QString name = settings.value(QStringLiteral("Flatpak Ref/Name")).toString();
-    const QString branch = settings.value(QStringLiteral("Flatpak Ref/Branch")).toString();
-    const QString remoteName = settings.value(QStringLiteral("Flatpak Ref/SuggestRemoteName")).toString();
-    const bool isRuntime = settings.value(QStringLiteral("Flatpak Ref/IsRuntime")).toBool();
+    const QString name = settings.value(QLatin1StringView("Flatpak Ref/Name")).toString();
+    const QString branch = settings.value(QLatin1StringView("Flatpak Ref/Branch")).toString();
+    const QString remoteName = settings.value(QLatin1StringView("Flatpak Ref/SuggestRemoteName")).toString();
+    const bool isRuntime = settings.value(QLatin1StringView("Flatpak Ref/IsRuntime")).toBool();
 
     AppStream::Component asComponent;
-    asComponent.addUrl(AppStream::Component::UrlKindHomepage, settings.value(QStringLiteral("Flatpak Ref/Homepage")).toString());
-    asComponent.setDescription(settings.value(QStringLiteral("Flatpak Ref/Description")).toString());
-    asComponent.setName(settings.value(QStringLiteral("Flatpak Ref/Title")).toString());
-    asComponent.setSummary(settings.value(QStringLiteral("Flatpak Ref/Comment")).toString());
+    asComponent.addUrl(AppStream::Component::UrlKindHomepage, settings.value(QLatin1StringView("Flatpak Ref/Homepage")).toString());
+    asComponent.setDescription(settings.value(QLatin1StringView("Flatpak Ref/Description")).toString());
+    asComponent.setName(settings.value(QLatin1StringView("Flatpak Ref/Title")).toString());
+    asComponent.setSummary(settings.value(QLatin1StringView("Flatpak Ref/Comment")).toString());
     asComponent.setId(name);
 
     AppStream::Bundle b;
@@ -775,8 +785,8 @@ AppStream::Component fetchComponentFromRemote(const QSettings &settings, GCancel
     g_autoptr(FlatpakRemote) tempRemote = flatpak_remote_new(remoteName.toUtf8().constData());
     populateRemote(tempRemote,
                    remoteName,
-                   settings.value(QStringLiteral("Flatpak Ref/Url")).toString(),
-                   settings.value(QStringLiteral("Flatpak Ref/GPGKey")).toString());
+                   settings.value(QLatin1StringView("Flatpak Ref/Url")).toString(),
+                   settings.value(QLatin1StringView("Flatpak Ref/GPGKey")).toString());
     if (!flatpak_installation_modify_remote(tempInstallation, tempRemote, cancellable, &localError)) {
         qCDebug(LIBDISCOVER_BACKEND_FLATPAK_LOG) << "error adding temporary remote" << localError->message;
         return asComponent;
@@ -802,23 +812,8 @@ AppStream::Component fetchComponentFromRemote(const QSettings &settings, GCancel
     const QString appstreamLocation = path + "/appstream/"_L1 + remoteName + '/'_L1 + QString::fromUtf8(flatpak_get_default_arch()) + "/active"_L1;
 
     AppStream::Pool pool;
-#ifdef APPSTREAM_NEW_POOL_API
     pool.setLoadStdDataLocations(false);
-#if ASQ_CHECK_VERSION(0, 16, 0)
     pool.addExtraDataLocation(appstreamLocation, AppStream::Metadata::FormatStyleCatalog);
-#else
-    pool.addExtraDataLocation(appstreamLocation, AppStream::Metadata::FormatStyleCollection);
-#endif
-#else
-    pool.clearMetadataLocations();
-    pool.addMetadataLocation(appstreamLocation);
-    pool.setFlags(AppStream::Pool::FlagReadCollection);
-    pool.setCacheFlags(AppStream::Pool::CacheFlagUseUser);
-
-    const QString subdir = flatpak_installation_get_id(tempInstallation) + QLatin1Char('/') + remoteName;
-    pool.setCacheLocation(QStandardPaths::writableLocation(QStandardPaths::CacheLocation) + "/flatpak-appstream-temp/" + subdir);
-    QDir().mkpath(pool.cacheLocation());
-#endif
 
     if (!pool.load()) {
         qCDebug(LIBDISCOVER_BACKEND_FLATPAK_LOG) << "error loading pool" << pool.lastError();
@@ -843,11 +838,11 @@ void FlatpakBackend::addAppFromFlatpakRef(const QUrl &url, ResultsStream *stream
 {
     Q_ASSERT(url.isLocalFile());
     QSettings settings(url.toLocalFile(), QSettings::NativeFormat);
-    const QString refurl = settings.value(QStringLiteral("Flatpak Ref/Url")).toString();
-    const QString name = settings.value(QStringLiteral("Flatpak Ref/Name")).toString();
-    const QString remoteName = settings.value(QStringLiteral("Flatpak Ref/SuggestRemoteName")).toString();
-    const QString branch = settings.value(QStringLiteral("Flatpak Ref/Branch")).toString();
-    const bool isRuntime = settings.value(QStringLiteral("Flatpak Ref/IsRuntime")).toBool();
+    const QString refurl = settings.value(QLatin1StringView("Flatpak Ref/Url")).toString();
+    const QString name = settings.value(QLatin1StringView("Flatpak Ref/Name")).toString();
+    const QString remoteName = settings.value(QLatin1StringView("Flatpak Ref/SuggestRemoteName")).toString();
+    const QString branch = settings.value(QLatin1StringView("Flatpak Ref/Branch")).toString();
+    const bool isRuntime = settings.value(QLatin1StringView("Flatpak Ref/IsRuntime")).toBool();
     g_autoptr(GError) error = nullptr;
 
     // If we already added the remote, just go with it
@@ -882,7 +877,7 @@ void FlatpakBackend::addAppFromFlatpakRef(const QUrl &url, ResultsStream *stream
     }
 
     AppStream::Component asComponent = fetchComponentFromRemote(settings, m_cancellable);
-    const QString iconUrl = settings.value(QStringLiteral("Flatpak Ref/Icon")).toString();
+    const QString iconUrl = settings.value(QLatin1StringView("Flatpak Ref/Icon")).toString();
     if (!iconUrl.isEmpty()) {
         AppStream::Icon icon;
         icon.setKind(AppStream::Icon::KindRemote);
@@ -895,13 +890,13 @@ void FlatpakBackend::addAppFromFlatpakRef(const QUrl &url, ResultsStream *stream
     resource->setResourceFile(url);
     resource->setResourceLocation(QUrl(refurl));
     resource->setOrigin(remoteName);
-    resource->setDisplayOrigin(remote ? QString::fromUtf8(flatpak_remote_get_title(remote)) : QString());
+    resource->setDisplayOrigin(remote ? copyAndFree(flatpak_remote_get_title(remote)) : QString());
     resource->setFlatpakName(name);
     resource->setArch(QString::fromUtf8(flatpak_get_default_arch()));
     resource->setBranch(branch);
     resource->setType(isRuntime ? FlatpakResource::Runtime : FlatpakResource::DesktopApp);
 
-    QUrl runtimeUrl = QUrl(settings.value(QStringLiteral("Flatpak Ref/RuntimeRepo")).toString());
+    QUrl runtimeUrl = QUrl(settings.value(QLatin1StringView("Flatpak Ref/RuntimeRepo")).toString());
     auto refSource = QSharedPointer<FlatpakSource>::create(this, preferredInstallation());
     resource->setTemporarySource(refSource);
     m_flatpakSources += refSource;
@@ -955,9 +950,9 @@ void FlatpakBackend::addSourceFromFlatpakRepo(const QUrl &url, ResultsStream *st
     Q_ASSERT(url.isLocalFile());
     QSettings settings(url.toLocalFile(), QSettings::NativeFormat);
 
-    const QString gpgKey = settings.value(QStringLiteral("Flatpak Repo/GPGKey")).toString();
-    const QString title = settings.value(QStringLiteral("Flatpak Repo/Title")).toString();
-    const QString repoUrl = settings.value(QStringLiteral("Flatpak Repo/Url")).toString();
+    const QString gpgKey = settings.value(QLatin1StringView("Flatpak Repo/GPGKey")).toString();
+    const QString title = settings.value(QLatin1StringView("Flatpak Repo/Title")).toString();
+    const QString repoUrl = settings.value(QLatin1StringView("Flatpak Repo/Url")).toString();
 
     if (gpgKey.isEmpty() || title.isEmpty() || repoUrl.isEmpty()) {
         return;
@@ -968,13 +963,13 @@ void FlatpakBackend::addSourceFromFlatpakRepo(const QUrl &url, ResultsStream *st
     }
 
     AppStream::Component asComponent;
-    asComponent.addUrl(AppStream::Component::UrlKindHomepage, settings.value(QStringLiteral("Flatpak Repo/Homepage")).toString());
-    asComponent.setSummary(settings.value(QStringLiteral("Flatpak Repo/Comment")).toString());
-    asComponent.setDescription(settings.value(QStringLiteral("Flatpak Repo/Description")).toString());
+    asComponent.addUrl(AppStream::Component::UrlKindHomepage, settings.value(QLatin1StringView("Flatpak Repo/Homepage")).toString());
+    asComponent.setSummary(settings.value(QLatin1StringView("Flatpak Repo/Comment")).toString());
+    asComponent.setDescription(settings.value(QLatin1StringView("Flatpak Repo/Description")).toString());
     asComponent.setName(title);
-    asComponent.setId(settings.value(QStringLiteral("Flatpak Repo/Title")).toString());
+    asComponent.setId(settings.value(QLatin1StringView("Flatpak Repo/Title")).toString());
 
-    const QString iconUrl = settings.value(QStringLiteral("Flatpak Repo/Icon")).toString();
+    const QString iconUrl = settings.value(QLatin1StringView("Flatpak Repo/Icon")).toString();
     if (!iconUrl.isEmpty()) {
         AppStream::Icon icon;
         icon.setKind(AppStream::Icon::KindRemote);
@@ -984,10 +979,10 @@ void FlatpakBackend::addSourceFromFlatpakRepo(const QUrl &url, ResultsStream *st
 
     auto resource = new FlatpakResource(asComponent, preferredInstallation(), this);
     // Use metadata only for stuff which are not common for all resources
-    resource->addMetadata(QStringLiteral("gpg-key"), gpgKey);
-    resource->addMetadata(QStringLiteral("repo-url"), repoUrl);
-    resource->setBranch(settings.value(QStringLiteral("Flatpak Repo/DefaultBranch")).toString());
-    resource->setFlatpakName(url.fileName().remove(QStringLiteral(".flatpakrepo")));
+    resource->addMetadata(QLatin1StringView("gpg-key"), gpgKey);
+    resource->addMetadata(QLatin1StringView("repo-url"), repoUrl);
+    resource->setBranch(settings.value(QLatin1StringView("Flatpak Repo/DefaultBranch")).toString());
+    resource->setFlatpakName(url.fileName().remove(QLatin1StringView(".flatpakrepo")));
     resource->setType(FlatpakResource::Source);
 
     g_autoptr(FlatpakRemote) repo =
@@ -1019,7 +1014,7 @@ bool FlatpakBackend::loadAppsFromAppstreamData(FlatpakInstallation *flatpakInsta
 {
     Q_ASSERT(flatpakInstallation);
 
-    GError *error = nullptr;
+    g_autoptr(GError) error = nullptr;
     g_autoptr(GPtrArray) remotes = flatpak_installation_list_remotes(flatpakInstallation, m_cancellable, &error);
     if (!remotes) {
         qCWarning(LIBDISCOVER_BACKEND_FLATPAK_LOG) << "failed to list remotes" << error->message;
@@ -1055,7 +1050,7 @@ void FlatpakBackend::unloadRemote(FlatpakInstallation *installation, FlatpakRemo
 {
     acquireFetching(true);
     for (auto it = m_flatpakSources.begin(); it != m_flatpakSources.end();) {
-        if ((*it)->url() == QString::fromUtf8(flatpak_remote_get_url(remote)) && (*it)->installation() == installation) {
+        if ((*it)->url() == copyAndFree(flatpak_remote_get_url(remote)) && (*it)->installation() == installation) {
             qCDebug(LIBDISCOVER_BACKEND_FLATPAK_LOG) << "unloading remote" << (*it) << remote;
             it = m_flatpakSources.erase(it);
         } else {
@@ -1100,23 +1095,8 @@ void FlatpakBackend::createPool(QSharedPointer<FlatpakSource> source)
     AppStream::Pool *pool = new AppStream::Pool;
     acquireFetching(true);
 
-#ifdef APPSTREAM_NEW_POOL_API
     pool->setLoadStdDataLocations(false);
-#if ASQ_CHECK_VERSION(0, 16, 0)
     pool->addExtraDataLocation(appstreamDirPath, AppStream::Metadata::FormatStyleCatalog);
-#else
-    pool->addExtraDataLocation(appstreamDirPath, AppStream::Metadata::FormatStyleCollection);
-#endif
-#else
-    pool->clearMetadataLocations();
-    pool->addMetadataLocation(appstreamDirPath);
-    pool->setFlags(AppStream::Pool::FlagReadCollection);
-    pool->setCacheFlags(AppStream::Pool::CacheFlagUseUser);
-
-    const QString subdir = flatpak_installation_get_id(source->installation()) + QLatin1Char('/') + source->name();
-    pool->setCacheLocation(QStandardPaths::writableLocation(QStandardPaths::CacheLocation) + "/flatpak-appstream/" + subdir);
-    QDir().mkpath(pool->cacheLocation());
-#endif
 
     const auto loadDone = [this, source, pool](bool result) {
         source->m_pool = pool;
@@ -1130,20 +1110,17 @@ void FlatpakBackend::createPool(QSharedPointer<FlatpakSource> source)
         acquireFetching(false);
     };
 
-    // We do not want to block the GUI for very long, so we load appstream pools via queued
-    // invocation both for the actual load and then for the post-load state changes.
-    QMetaObject::invokeMethod(
-        this,
-        [this, pool, loadDone] {
-            auto result = pool->load();
-            QMetaObject::invokeMethod(
-                this,
-                [loadDone, result] {
-                    loadDone(result);
-                },
-                Qt::QueuedConnection);
-        },
-        Qt::QueuedConnection);
+    connect(pool, &AppStream::Pool::loadFinished, this, [this, loadDone](bool success) {
+        // We do not want to block the GUI for very long, so we load appstream pools via queued
+        // invocation for the post-load state changes.
+        QMetaObject::invokeMethod(
+            this,
+            [loadDone, success] {
+                loadDone(success);
+            },
+            Qt::QueuedConnection);
+    });
+    pool->loadAsync();
 }
 
 QSharedPointer<FlatpakSource> FlatpakBackend::integrateRemote(FlatpakInstallation *flatpakInstallation, FlatpakRemote *remote)
@@ -1152,7 +1129,7 @@ QSharedPointer<FlatpakSource> FlatpakBackend::integrateRemote(FlatpakInstallatio
     m_sources->addRemote(remote, flatpakInstallation);
     const auto matchRemote = [this, flatpakInstallation, remote](const auto &sources) -> QSharedPointer<FlatpakSource> {
         for (auto source : sources) {
-            if (source->url() == QString::fromUtf8(flatpak_remote_get_url(remote)) && source->installation() == flatpakInstallation
+            if (source->url() == copyAndFree(flatpak_remote_get_url(remote)) && source->installation() == flatpakInstallation
                 && source->name() == QString::fromUtf8(flatpak_remote_get_name(remote))) {
                 createPool(source);
                 if (source->remote() != remote) {
@@ -1259,7 +1236,7 @@ bool FlatpakBackend::updateAppMetadata(FlatpakResource *resource)
         return true;
     }
 
-    const QString path = resource->installPath() + QStringLiteral("/metadata");
+    const QString path = resource->installPath() + QLatin1StringView("/metadata");
 
     if (QFile::exists(path)) {
         return updateAppMetadata(resource, path);
@@ -1602,7 +1579,7 @@ ResultsStream *FlatpakBackend::search(const AbstractResourcesBackend::Filters &f
 
                             const QByteArray parentId(id.constData(), id.lastIndexOf(QLatin1Char('.')));
 
-                            // We need to bruteforce the API as we don't know from here if it's app or runtime 🙄
+                            // We need to bruteforce the API as we don't know from here if ref's parent is an app or a runtime 🙄
                             auto parentRef = flatpak_installation_get_installed_ref(installation,
                                                                                     FLATPAK_REF_KIND_APP,
                                                                                     parentId.constData(),
@@ -1611,6 +1588,7 @@ ResultsStream *FlatpakBackend::search(const AbstractResourcesBackend::Filters &f
                                                                                     cancellable,
                                                                                     &localError);
                             if (!parentRef) {
+                                g_clear_error(&localError);
                                 parentRef = flatpak_installation_get_installed_ref(installation,
                                                                                    FLATPAK_REF_KIND_RUNTIME,
                                                                                    parentId.constData(),
@@ -1723,17 +1701,15 @@ ResultsStream *FlatpakBackend::search(const AbstractResourcesBackend::Filters &f
 
                     QList<FlatpakResource *> resources;
                     if (source->m_pool) {
-                        const auto components = [&]() {
+                        const auto components = co_await [](const auto &filter, const auto &source) -> QCoro::Task<AppStream::ComponentBox> {
                             if (!filter.search.isEmpty()) {
-                                return source->m_pool->search(filter.search);
+                                co_return source->m_pool->search(filter.search);
                             }
-#if ASQ_CHECK_VERSION(0, 15, 6)
                             if (filter.category) {
-                                return AppStreamUtils::componentsByCategories(source->m_pool, filter.category, AppStream::Bundle::KindFlatpak);
+                                co_return co_await AppStreamUtils::componentsByCategoriesTask(source->m_pool, filter.category, AppStream::Bundle::KindFlatpak);
                             }
-#endif
-                            return source->m_pool->components();
-                        }();
+                            co_return source->m_pool->components();
+                        }(filter, source);
 
                         resources.reserve(components.size());
                         for (const auto &component : components) {
@@ -1749,7 +1725,8 @@ ResultsStream *FlatpakBackend::search(const AbstractResourcesBackend::Filters &f
                         FLATPAK_BACKEND_YIELD;
 
                         const bool matchById = resource->appstreamId().compare(filter.search, Qt::CaseInsensitive) == 0;
-                        if (resource->type() == AbstractResource::Technical && filter.state != AbstractResource::Upgradeable && !matchById) {
+                        // Note: FlatpakResource can not have type == System
+                        if (resource->type() == AbstractResource::ApplicationSupport && filter.state != AbstractResource::Upgradeable && !matchById) {
                             continue;
                         }
 
@@ -1981,8 +1958,8 @@ FlatpakRemote *FlatpakBackend::installSource(FlatpakResource *resource)
     auto remote = flatpak_remote_new(resource->flatpakName().toUtf8().constData());
     populateRemote(remote,
                    resource->comment(),
-                   resource->getMetadata(QStringLiteral("repo-url")).toString(),
-                   resource->getMetadata(QStringLiteral("gpg-key")).toString());
+                   resource->getMetadata(QLatin1StringView("repo-url")).toString(),
+                   resource->getMetadata(QLatin1StringView("gpg-key")).toString());
     if (!resource->branch().isEmpty()) {
         flatpak_remote_set_default_branch(remote, resource->branch().toUtf8().constData());
     }
@@ -2061,6 +2038,7 @@ Transaction *FlatpakBackend::removeApplication(AbstractResource *app)
     connect(transaction, &FlatpakJobTransaction::repositoriesAdded, this, &FlatpakBackend::checkRepositories);
     connect(transaction, &FlatpakJobTransaction::statusChanged, this, [this, resource](Transaction::Status status) {
         if (status == Transaction::Status::DoneStatus) {
+            updateAppState(resource);
             updateAppSize(resource);
         }
     });
