@@ -8,6 +8,7 @@
 
 #include "PackageKitBackend.h"
 #include "PackageKitResource.h"
+#include <QIcon>
 
 class AppPackageKitResource : public PackageKitResource
 {
@@ -45,12 +46,13 @@ public:
     QSet<QString> alternativeAppstreamIds() const override;
     bool isCritical() const override;
 
-    QString contentRatingText() const override;
     QString contentRatingDescription() const override;
-    ContentIntensity contentRatingIntensity() const override;
     uint contentRatingMinimumAge() const override;
+    [[nodiscard]] bool hasResolvedIcon() const override;
+    void resolveIcon() override;
 
 private:
     const AppStream::Component m_appdata;
     mutable QString m_name;
+    std::optional<QIcon> m_icon;
 };
